// istanbul ignore next
'use strict';

var _interopRequireDefault = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

// istanbul ignore next

var _toArray = function (arr) { return Array.isArray(arr) ? arr : Array.from(arr); };

// istanbul ignore next

var _classCallCheck = function (instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } };

// istanbul ignore next

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _list = require('postcss/lib/list');

var _list2 = _interopRequireDefault(_list);

var _rfc = require('reduce-function-call');

var _rfc2 = _interopRequireDefault(_rfc);

var _parsePath2 = require('parse-filepath');

var _parsePath3 = _interopRequireDefault(_parsePath2);

var _yaml = require('js-yaml');

var _yaml2 = _interopRequireDefault(_yaml);

var reMap = /((?:map)\()(.*)(\))/;
var defaultMap = 'config';

var MapParser = (function () {
  function MapParser() {
    var _ref = arguments[0] === undefined ? {} : arguments[0];

    var _ref$basePath = _ref.basePath;
    var basePath = _ref$basePath === undefined ? process.cwd() : _ref$basePath;
    var _ref$maps = _ref.maps;
    var maps = _ref$maps === undefined ? [] : _ref$maps;

    _classCallCheck(this, MapParser);

    this.basePath = basePath;
    this.maps = {};

    maps.forEach(this.getMap.bind(this));
  }

  _createClass(MapParser, [{
    key: 'processDecl',

    /**
     * Parse and replace maps in declarations values.
     * @param {Object} decl
     */
    value: function processDecl(decl) {
      var _this = this;

      if (!reMap.test(decl.value)) {
        return;
      }decl.value = _rfc2['default'](decl.value, 'map', function (body) {
        return _this.getValue(_list2['default'].comma(body));
      });
    }
  }, {
    key: 'processAtRule',

    /**
     * Parse and proccess at-rules.
     * @param {Object} rule
     */
    value: function processAtRule(rule) {
      if (rule.name === 'map') {
        return this.replaceAtRuleBlock(rule);
      }if (reMap.test(rule.params)) {
        return this.replaceAtRuleParam(rule);
      }
    }
  }, {
    key: 'replaceAtRuleBlock',

    /**
     * Get and print at-rules map declarations.
     * @param {Object} rule
     */
    value: function replaceAtRuleBlock(rule) {
      var map = this.getValue(_list2['default'].space(rule.params));

      Object.keys(map).forEach(function (prop) {
        rule.parent.insertBefore(rule, { prop: prop, value: map[prop] });
      });

      rule.removeSelf();
    }
  }, {
    key: 'replaceAtRuleParam',

    /**
     * Parse and replace maps in at-rules parameters.
     * @param {Object} rule
     */
    value: function replaceAtRuleParam(rule) {
      var _this2 = this;

      rule.params = _rfc2['default'](rule.params, 'map', function (body) {
        return _this2.getValue(_list2['default'].comma(body));
      });
    }
  }, {
    key: 'getValue',

    /**
     * Get value from a deep nested object properties.
     * @param {Array} args
     * @return {*}
     */
    value: function getValue(args) {
      var _args = _toArray(args);

      var name = _args[0];

      var props = _args.slice(1);

      var shortcutMap = this.useShortcutMap(name);

      if (shortcutMap) {
        name = shortcutMap;
        props = args;
      }

      var value = this.getMap(name);

      props.forEach(function (prop) {
        value = value[prop];
      });

      return value;
    }
  }, {
    key: 'useShortcutMap',

    /**
     * Get map name usable with the short syntax.
     * @param {String} name
     * @return {Boolean|String}
     */
    value: function useShortcutMap(name) {
      if (name in this.maps) {
        return false;
      }if (defaultMap in this.maps) {
        return defaultMap;
      }var names = Object.keys(this.maps);
      if (names.length === 1) {
        return names[0];
      }
    }
  }, {
    key: 'parseFile',

    /**
     * Load YAML or JSON from given file.
     *
     * @param {String} file
     * @return {Object}
     */
    value: function parseFile(file) {
      try {
        var filePath = _path2['default'].resolve(this.basePath, file);
        return _yaml2['default'].safeLoad(_fs2['default'].readFileSync(filePath, 'utf-8'));
      } catch (e) {
        console.error(e);
      }
    }
  }, {
    key: 'getMap',

    /**
     * Get a cached map or parse it.
     *
     * @param {String} map
     * @return {Object}
     */
    value: function getMap(map) {
      var _parsePath = _parsePath3['default'](map);

      var name = _parsePath.name;
      var basename = _parsePath.basename;

      return this.maps[name] || (this.maps[name] = this.parseFile(basename));
    }
  }]);

  return MapParser;
})();

exports['default'] = _postcss2['default'].plugin('postcss-map', function (opts) {
  var parser = new MapParser(opts);

  return function (css) {
    css.eachDecl(parser.processDecl.bind(parser));
    css.eachAtRule(parser.processAtRule.bind(parser));
  };
});
module.exports = exports['default'];